{
   Unit Name      : MILFieldList.pas
   Component Name : TMILFieldList
   Author         : Larry J. Rutledge
   Created        : May 26, 1999
   Modified       : June 4, 1999

   Description    : This component was designed for an application where it
                    was necessary to represent individual database tables and
                    their fields. The application also required that dragging
                    and dropping between the components implement a "link" on
                    the selected fields (Items).

                    The requirements were met with this component. It's
                    appearance is that of a small Windows form. Using the
                    Moveable and Sizable properties it is possible to allow the
                    component to be moved and re-sized on the form at run-time,
                    much the same way that a Windows form is moved and re-sized.

                    The "caption bar" is where the database table name is
                    displayed, however any text assigned to the Caption property
                    will be displayed as the components "caption" or "title".

                    The remainder of the component consists of a ListBox that
                    was used to display the database table's fields. Clicking
                    on one of these "fields" and dragging it to a "field" on
                    another component would generate an OnLink event in the
                    destination component. This event indicates the Source and
                    Destination components and the Items and ItemIndex
                    properties of each can be used to determine the "fields" to
                    link.

   Property List  : This component adds the following properties:

                    About         : Displays the About form for this component.
                                    Only available at design-time.
                    Active        : Indicates whether the component is Active
                                    or Inactive. Used to determine the Color
                                    and font settings to use when painting the
                                    "caption bar".
                    ActiveColor   : Indicates the color to use when painting the
                                    "caption bar" if the component is Active.
                                    (Has the focus).
                    ActiveFont    : Indicates the font to use when painting the
                                    "caption bar" if the component is Active.
                    Caption       : The text to display in the "caption bar".
                    Font          : The font to use when drawing the Items
                                    (fields) in the ListBox.
                    InactiveColor : Indicates the color to use when painting the
                                    "caption bar" if the component is not Active
                    InactiveFont  : Indicates the font to use when painting the
                                    "caption bar" if the component is not Active
                    Items         : The list of items(fields) to display.
                    Moveable      : Indicates whether the component can be
                                    moved at run-time by dragging the "Caption
                                    bar".
                    ScrollBar     : Indicates the scroll bar to use when the
                                    Items (fields) list is too long or wide for
                                    the current dimensions of the component. Can
                                    be sbHorizontal, sbVertical, sbBoth, or
                                    sbNone.
                    Sizeable      : Indicates whether the component can be
                                    resized at run-time by dragging the borders.

   Important Note : No notes for this component

   History        : Version  Date         Author     Description
                    -------  ----------   ------     --------------------------
                    1.0.0    05/26/1999    LJR       Created initial component.

                    1.0.1    05/27/1999    LJR       Fixed Access Violation that
                                                     occurred when changing the
                                                     ActiveFont or InactiveFont
                                                     properties.

                    1.0.2    06/04/1999    LJR       Added FMoveable and
                                                     FSizeable properties to
                                                     allow the component user
                                                     to determine whether to
                                                     allow Moving and Sizing at
                                                     run-time.


   Proposed
   Revisions      : 1) Add AutoSize property
}

unit MILFieldList;

interface

uses
  MILAbout,
  Windows, Messages, DsgnIntf, SysUtils, Classes, Graphics, Controls, StdCtrls,
  ExtCtrls, Forms;

const
  _TITLE     = 'TMILFieldList';
  _VERSION   = '1.0.2';
  _AUTHOR    = 'Larry J. Rutledge';
  _COPYRIGHT = ' Copyright 1998 - 1999, Millennial Software';

type
  { TAboutProperty - A Property editor for the About property. Displays an    }
  {                  about window when the ellipses are clicked.              }
  TAboutProperty = class(TPropertyEditor)
  public
    function  GetValue      : string; override;
    function  GetAttributes : TPropertyAttributes; override;

    procedure Edit; override;
  end;

  // Custom Event type - describes parameter list for OnLink event
  TLinkEvent = procedure(Sender, Source: TObject) of object;

  // Options available in the scrollbar property
  TScrollBarType = (sbNone, sbVertical, sbHorizontal, sbBoth);

  TMILFieldList = class(TCustomControl)
  private
    // Property Variables
    FAbout         : TAboutProperty;

    FActive        : Boolean;
    FActiveColor   : TColor;
    FActiveFont    : TFont;
    FInactiveColor : TColor;
    FInactiveFont  : TFont;
    FItemIndex     : Integer;
    FItems         : TStrings;
    FMoveable      : Boolean;
    FScrollBar     : TScrollBarType;
    FSizeable      : Boolean;

    // Event Variables
    FOnLink        : TLinkEvent;
    FOnResize      : TNotifyEvent;

    // Access Methods

    // Mutator Methods
    procedure SetActive(Value: Boolean);
    procedure SetActiveColor(Value: TColor);
    procedure SetActiveFont(Value: TFont);
    procedure SetInactiveColor(Value: TColor);
    procedure SetInactiveFont(Value: TFont);
    procedure SetItems(Value: TStrings);
    procedure SetScrollBar(Value: TScrollBarType);
    procedure SetItemIndex(Value: Integer);

    // Internal processing
    procedure DrawFrame(CurrRect: TRect);
    procedure DrawCaption(CurrRect: TRect);
    procedure DrawListBox;

    // Internal events
    procedure ItemsChanged(Sender: TObject);
  protected
    CaptionRect : TRect;
    lstItemList : TListBox;

    // Events assigned to owned Components
      // Assigned to Caption fonts
    procedure FontChanged(Sender: TObject);

      // Assigned to ListBox
    procedure ItemListClick(Sender: TObject);
    procedure ItemListDragOver(Sender, Source: TObject; X, Y: Integer; State:
      TDragState; var Accept: Boolean);
    procedure ItemListDragDrop(Sender, Source: TObject; X, Y: Integer);

    // Overridden methods
    procedure DoEnter; override;
    procedure DoExit; override;

    // Windows messages
    procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
    procedure WMLButtonDown(var Msg: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMNCHitTest(var Msg: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Loaded; override;

    procedure Paint; override;

    // Run-time only property
    property ItemIndex : Integer
      read FItemIndex write SetItemIndex default -1;
  published
    // Custom properties
    property About: TAboutProperty
      read FAbout write FAbout;

    property Active: Boolean
      read FActive write SetActive default True;
    property ActiveColor: TColor
      read FActiveColor write SetActiveColor default clActiveCaption;
    property ActiveFont: TFont
      read FActiveFont write SetActiveFont;
    property InactiveColor: TColor
      read FInactiveColor write SetInactiveColor default clInactiveCaption;
    property InactiveFont: TFont
      read FInactiveFont write SetInactiveFont;
    property Items: TStrings
      read FItems write SetItems;
    property Moveable: Boolean
      read FMoveable write FMoveable default True;
    property ScrollBar: TScrollBarType
      read FScrollBar write SetScrollBar default sbVertical;
    property Sizeable: Boolean
      read FSizeable write FSizeable default True;

    // Custom events
    property OnLink: TLinkEvent
      read FOnLink write FOnLink;
    property OnResize: TNotifyEvent
      read FOnResize write FOnResize;

    // Inherited properties
    property Caption;
    property Color;
    property Cursor;
    property Enabled;
    property Font;
    property Hint;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;

    // Inherited events
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

{---------------------------------------------------------}
{ TAboutProperty - About Form Property Editor methods     }
{---------------------------------------------------------}

(******************************************************************************)
(* GetValue      -  Called when Delphi wants to display the value of the      *)
(*                  property in the Object Inspector.                         *)
(******************************************************************************)
function  TAboutProperty.GetValue : string;
begin
  Result := '(about)';
end;

(******************************************************************************)
(* GetAttributes -  Called when Delphi wants to know what attributes to apply *)
(*                  to the property.                                          *)
(******************************************************************************)
function  TAboutProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paReadOnly];
end;

(******************************************************************************)
(* Edit          -  Called when the user clicks the ellipses in the value     *)
(*                  field.                                                    *)
(******************************************************************************)
procedure TAboutProperty.Edit;
begin
  with TfrmMILAbout.Create(Application) do
  try
    SetLabels(_TITLE, _VERSION, _AUTHOR, _COPYRIGHT);
    ShowModal;
  finally
    Free;
  end;
end;

{---------------------------------------------------------}
{ TMILFieldList - Component definition                    }
{---------------------------------------------------------}
constructor TMILFieldList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // Set default values for custom properties and create instances of Objects
  // owned by component
  FActive                := True;

  FActiveColor           := clActiveCaption;
  FInactiveColor         := clInactiveCaption;

  FActiveFont            := TFont.Create;
  FActiveFont.Color      := clCaptionText;
  FActiveFont.Name       := 'Arial';
  FActiveFont.Size       := 10;
  FActiveFont.Style      := [];
  FActiveFont.OnChange   := FontChanged;

  FInactiveFont          := TFont.Create;
  FInactiveFont.Color    := clInactiveCaptionText;
  FInactiveFont.Name     := 'Arial';
  FInactiveFont.Size     := 10;
  FInactiveFont.Style    := [];
  FInactiveFont.OnChange := FontChanged;

  FItemIndex             := -1;

  FItems                 := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChanged;

  lstItemList            := TListBox.Create(Self);
  lstItemList.DragMode   := dmAutomatic;
  lstItemList.Parent     := Self;
  lstItemList.OnClick    := ItemListClick;
  lstItemList.OnDragOver := ItemListDragOver;
  lstItemList.OnDragDrop := ItemListDragDrop;

  FMoveable              := True;
  FScrollBar             := sbVertical;
  FSizeable              := True;

  // Initialize the Caption rectangle for paint methods
  CaptionRect            := Rect(0, 0, 0, 0);

  Color                  := clBtnFace;
  Width                  := 150;
  Height                 := 200;

  ControlStyle := ControlStyle + [csSetCaption];
end;

destructor TMILFieldList.Destroy;
begin
  // Free objects created by component
  FItems.Free;

  FActiveFont.Free;
  FInactiveFont.Free;

  inherited Destroy;
end;

{---------------------------------------------------------}
{ Accessor Methods - Retrieves the value of a property.   }
{                    Allows for the value to manipulated  }
{                    or calculated before returning.      }
{---------------------------------------------------------}



{---------------------------------------------------------}
{ Mutator Methods - Assigns a new value to a property.    }
{                   Allows for the value to manipulated   }
{                   or calculated before storing.         }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ SetActive       - Sets the value of FActive and         }
{                   invalidates only the caption area for }
{                   re-painting. The value of FActive is  }
{                   used during painting to determine     }
{                   whether to use the Active or Inactive }
{                   Font and Color.                       }
{---------------------------------------------------------}
procedure TMILFieldList.SetActive(Value: Boolean);
var
  TempRect: TRect;

begin
  if FActive <> Value then
  begin
    FActive := Value;

    TempRect := CaptionRect;
    InvalidateRect(Handle, @TempRect, False);

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ SetActiveColor  - Sets the value of FActiveColor and    }
{                   invalidates only the caption area for }
{                   re-painting. The value of FActiveColor}
{                   is used when painting while FActive   }
{                   True.                                 }
{---------------------------------------------------------}
procedure TMILFieldList.SetActiveColor(Value: TColor);
var
  TempRect: TRect;

begin
  if FActiveColor <> Value then
  begin
    FActiveColor := Value;

    TempRect := CaptionRect;
    InvalidateRect(Handle, @TempRect, False);

    Repaint;
  end;
end;

procedure TMILFieldList.SetActiveFont(Value: TFont);
var
  TempRect: TRect;

begin
  if FActiveFont <> Value then
  begin
    FActiveFont.Assign(Value);

    TempRect := CaptionRect;
    InvalidateRect(Handle, @TempRect, False);

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ SetInactiveColor- Sets the value of FInactiveColor and  }
{                   invalidates only the caption area for }
{                   re-painting. The value of             }
{                   FInactiveColor is used when painting  }
{                   while FActive is False.               }
{---------------------------------------------------------}
procedure TMILFieldList.SetInactiveColor(Value: TColor);
var
  TempRect: TRect;

begin
  if FInactiveColor <> Value then
  begin
    FInactiveColor := Value;

    TempRect := CaptionRect;
    InvalidateRect(Handle, @TempRect, False);

    Repaint;
  end;
end;

procedure TMILFieldList.SetInactiveFont(Value: TFont);
var
  TempRect: TRect;

begin
  if FInactiveFont <> Value then
  begin
    FInactiveFont.Assign(Value);

    TempRect := CaptionRect;
    InvalidateRect(Handle, @TempRect, False);

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ SetItems        - Assigns the FItems property to the    }
{                   Items property in the owned ListBox.  }
{---------------------------------------------------------}
procedure TMILFieldList.SetItems(Value: TStrings);
begin
  if FItems <> Value then
  begin
    FItems.Assign(Value);
    lstItemList.Items.Assign(Value);

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ SetScrollBar    - Sets the value of FScrollBar and      }
{                   forces a re-paint. FScrollBar is used }
{                   while painting to determine whether   }
{                   or not to display the Vertical and/or }
{                   Horizontal scroll bars in the ListBox.}
{---------------------------------------------------------}
procedure TMILFieldList.SetScrollBar(Value: TScrollBarType);
begin
  if FScrollBar <> Value then
  begin
    FScrollBar := Value;

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ SetItemIndex    - Sets the value of FItemIndex and the  }
{                   ItemIndex on the owned TListBox. Then }
{                   forces a repaint to show the selected }
{                   option.                               }
{---------------------------------------------------------}
procedure TMILFieldList.SetItemIndex(Value: Integer);
begin
  if FItemIndex <> Value then
  begin
    FItemIndex := Value;
    lstItemList.ItemIndex := Value;

    Repaint;
  end;
end;

{---------------------------------------------------------}
{ Internal methods used by Items (TStringList).           }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ ItemsChanged    - Set to the OnChange event of FItems.  }
{                   Should trigger if the Items list      }
{                   changes. As a result, the TListBox    }
{                   can be updated to reflect the changes.}
{                                                         }
{                   (Note: This fixes a bug where strings }
{                    added at run-time don't appear in the}
{                    TListBox.)                           }
{---------------------------------------------------------}
procedure TMILFieldList.ItemsChanged(Sender: TObject);
begin
  lstItemList.Items.Clear;
  lstItemList.Items.Assign(Self.Items);

  Repaint;
end;

{---------------------------------------------------------}
{ Embedded Class Methods - Methods assigned to events of  }
{                          owned controls.                }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ Active and Inactive Font - When they are changed the    }
{                            caption needs to be redrawn. }
{---------------------------------------------------------}
procedure TMILFieldList.FontChanged(Sender: TObject);
var
  TempRect: TRect;

begin
  TempRect := CaptionRect;
  InvalidateRect(Handle, @TempRect, False);

  Repaint;
end;

{---------------------------------------------------------}
{ ItemList Click  - When the ListBox is clicked, the      }
{                   selected item in the list needs to be }
{                   also selected in the internal FItems  }
{                   property.                             }
{---------------------------------------------------------}
procedure TMILFieldList.ItemListClick(Sender: TObject);
begin
  ItemIndex := lstItemList.ItemIndex;
end;

{---------------------------------------------------------}
{ ItemList DragOver - During a Drag-Drop operation, the   }
{                     component has to decide whether to  }
{                     accept what is being dropped or not.}
{---------------------------------------------------------}
procedure TMILFieldList.ItemListDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := False;

  if (TComponent(Source).Owner is TMILFieldList) and (Sender <> Source) then
  begin
    Accept := True;

    if TComponent(Sender).Owner is TMILFieldList then
    begin
      if TListBox(Sender).ItemAtPos(Point(X, Y), True) > -1 then
      begin
        TListBox(Sender).ItemIndex := TListBox(Sender).ItemAtPos(Point(X, Y), True);

        ItemIndex := TListBox(Sender).ItemIndex;
      end;
    end;
  end;
end;

{---------------------------------------------------------}
{ ItemList DropDrop - When the Drag-Drop operation is     }
{                     completed, the OnLink event has to  }
{                     be fired to indicate the link       }
{                     operation has occured.              }
{---------------------------------------------------------}
procedure TMILFieldList.ItemListDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  if (TListBox(Sender).Owner is TMILFieldList) and (TListBox(Source).Owner is
    TMILFieldList) then
  begin
    if Assigned(OnLink) then
      OnLink(TListBox(Sender).Owner, TListBox(Source).Owner);
  end;
end;

{---------------------------------------------------------}
{ Overridden Parent Class Methods                         }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ Loaded          - When the component is loaded from the }
{                   stream, if the Items property had any }
{                   stored values they need to be assigned}
{                   to the internal ListBox and displayed.}
{---------------------------------------------------------}
procedure TMILFieldList.Loaded;
begin
  lstItemList.Items.Assign(FItems);
end;

{---------------------------------------------------------}
{ Internal methods used by Paint.                         }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ DrawFrame       - Takes a TRect of the component and    }
{                   draws a 3D raised border around it.   }
{---------------------------------------------------------}
procedure TMILFieldList.DrawFrame(CurrRect: TRect);
var
  OldBrushStyle: TBrushStyle;
  OldPenColor:   TColor;
  OldPenStyle:   TPenStyle;

begin
  with Canvas do
  begin
    OldPenColor := Pen.Color;
    OldPenStyle := Pen.Style;
    OldBrushStyle := Brush.Style;
    Brush.Style   := bsClear;

    Pen.Style := psSolid;
    Pen.Color := clSilver;
    MoveTo(CurrRect.Left, CurrRect.Bottom);
    LineTo(CurrRect.Left, CurrRect.Top);
    LineTo(CurrRect.Right, CurrRect.Top);

    Pen.Color := clWhite;
    MoveTo(CurrRect.Left + 1, CurrRect.Bottom - 1);
    LineTo(CurrRect.Left + 1, CurrRect.Top + 1);
    LineTo(CurrRect.Right - 1, CurrRect.Top + 1);

    Pen.Color := clBlack;
    MoveTo(CurrRect.Left + 1, CurrRect.Bottom - 1);
    LineTo(CurrRect.Right - 1, CurrRect.Bottom - 1);
    LineTo(CurrRect.Right - 1, CurrRect.Top + 1);

    Pen.Color := clGray;
    MoveTo(CurrRect.Left + 1, CurrRect.Bottom - 2);
    LineTo(CurrRect.Right - 2, CurrRect.Bottom - 2);
    LineTo(CurrRect.Right - 2, CurrRect.Top + 2);

    Pen.Style   := OldPenStyle;
    Pen.Color   := OldPenColor;
    Brush.Style := OldBrushStyle
  end;
end;

{---------------------------------------------------------}
{ DrawCaption     - Takes a TRect of the component and    }
{                   calculates the TRect of the caption   }
{                   area. Then paints using either the    }
{                   Active or Inactive color, based on the}
{                   FActive property. Finaly, writes the  }
{                   the Caption in the caption area using }
{                   either the Active or Inactive font,   }
{                   based on the FActive proeprty.        }
{---------------------------------------------------------}
procedure TMILFieldList.DrawCaption(CurrRect: TRect);
var
  OldBrushColor: TColor;
  OldBrushStyle: TBrushStyle;

begin
  with Canvas do
  begin
    OldBrushColor := Brush.Color;
    OldBrushStyle := Brush.Style;

    Brush.Style := bsSolid;

    if Active then
    begin
      Brush.Color := FActiveColor;
      Font.Assign(FActiveFont);
    end else
    begin
      Brush.Color := FInactiveColor;
      Font.Assign(FInactiveFont);
    end;

    CaptionRect := Rect(CurrRect.Left + 4, CurrRect.Top + 4, CurrRect.Right - 4,
      CurrRect.Top + 4 + TextHeight('Caption') + 4);
    FillRect(CaptionRect);

    TextRect(CaptionRect, CaptionRect.Left + ((CaptionRect.Right -
      TextWidth(Caption)) div 2), CaptionRect.Top + 1, Caption);

    Brush.Color := OldBrushColor;
    Brush.Style := OldBrushStyle;
  end;
end;

{---------------------------------------------------------}
{ DrawListBox     - Positions the list box within the     }
{                   bounds of the component, including the}
{                   caption area. Sets the vertical and   }
{                   horizontal scrollbars visibility based}
{                   on the FScrollBar property.           }
{---------------------------------------------------------}
procedure TMILFieldList.DrawListBox;
var
  iCount   : Integer;
  MaxWidth : Integer;

begin
  lstItemList.Left   := 4;
  lstItemList.Top    := CaptionRect.Bottom + 4;
  lstItemList.Width  := Width - 8;
  lstItemList.Height := Height - CaptionRect.Bottom - 8;

  with lstItemList do
  begin
    case ScrollBar of
      sbNone       : begin
                       ShowScrollBar(Handle, SB_VERT, False);
                       SendMessage(Handle, LB_SETHORIZONTALEXTENT, 0, 0);
                     end;
      sbBoth       : begin
                       MaxWidth := 0;
                       for iCount := 0 to Items.Count - 1 do
                         if Canvas.TextWidth(Items[iCount]) > MaxWidth then
                           MaxWidth := Canvas.TextWidth(Items[iCount]);

                       SendMessage(Handle, LB_SETHORIZONTALEXTENT,
                         MaxWidth + 8, 0);
                     end;
      sbHorizontal : begin
                       MaxWidth := 0;
                       for iCount := 0 to Items.Count - 1 do
                         if Canvas.TextWidth(Items[iCount]) > MaxWidth then
                           MaxWidth := Canvas.TextWidth(Items[iCount]);

                       SendMessage(Handle, LB_SETHORIZONTALEXTENT,
                         MaxWidth + 8, 0);

                       ShowScrollBar(Handle, SB_VERT, False);
                     end;
      sbVertical   : begin
                       ShowScrollBar(Handle, SB_HORZ, False);
                     end;
    end;

    Refresh;
  end;
end;

{---------------------------------------------------------}
{ Paint - Called whenever the component needs to be drawn }
{         to the screen.                                  }
{---------------------------------------------------------}
procedure TMILFieldList.Paint;
var
  CurrRect: TRect;

begin
  CurrRect := GetClientRect;
  with Canvas do
  begin
    Brush.Color := Color;
    Brush.Style := bsSolid;

    FillRect(CurrRect);

    DrawFrame(CurrRect);
    DrawCaption(CurrRect);
    DrawListBox;
  end;
end;

{---------------------------------------------------------}
{ DoEnter - Called when the component receives the input  }
{           focus, either by being clicked on or being    }
{           tabbed to, or programmatically.               }
{---------------------------------------------------------}
procedure TMILFieldList.DoEnter;
begin
  inherited DoEnter;

  Active := True;
  lstItemList.SetFocus;
end;

{---------------------------------------------------------}
{ DoExit - Called when the component is losing the input  }
{          focus.                                         }
{---------------------------------------------------------}
procedure TMILFieldList.DoExit;
begin
  inherited DoExit;

  Active := False;
end;

{---------------------------------------------------------}
{ Windows Message Handlers                                }
{---------------------------------------------------------}

{---------------------------------------------------------}
{ CM_TEXTCHANGED - Called whenever a key is pressed in    }
{                  Caption property. Causes the caption   }
{                  to be modified, at design-time, on the }
{                  component at the same time as in the   }
{                  Object Inspector.                      }
{---------------------------------------------------------}
procedure TMILFieldList.CMTextChanged(var Msg: TMessage);
begin
  Repaint;
end;

{---------------------------------------------------------}
{ WM_LBUTTONDOWN - Called whenever the left mouse button  }
{                  is pressed, but before it is released).}
{                                                         }
{                  If the mouse is currently within the   }
{                  caption portion of the component, the  }
{                  component becomes draggable.           }
{---------------------------------------------------------}
procedure TMILFieldList.WMLButtonDown(var Msg: TWMLButtonDown);
const
  SC_DRAGMOVE = $F012;

begin
  if (PtInRect(CaptionRect, Point(Msg.XPos, Msg.YPos))) and (FMoveable) then
  begin
    DoEnter;

    SetCaptureControl(nil);
    Perform(WM_SYSCOMMAND, SC_DRAGMOVE, 0);
  end else
    DoEnter;

  if Assigned(OnMouseDown) then
    OnMouseDown(Self, mbLeft, [], Msg.XPos, Msg.YPos);
end;

{---------------------------------------------------------}
{ WM_NCHITTEST - Called whenever the mouse button is      }
{                while over the non-client portion of the }
{                component (i.e. the borders). Allows the }
{                component to be resized at run-time.     }
{---------------------------------------------------------}
procedure TMILFieldList.WMNCHitTest(var Msg: TWMNCHitTest);
var
  HitPt:     TPoint;
  CornerOfs: Integer;

begin
  Msg.Result := 0;

  if FSizeable then
  begin
    CornerOfs := CaptionRect.Bottom;

    HitPt := ScreenToClient(Point(Msg.XPos, Msg.YPos));

    if PtInRect(Rect(0, 0, CornerOfs, 3), HitPt) or
       PtInRect(Rect(0, 0, 3, CornerOfs), HitPt) then
    begin
      Msg.Result := htTopLeft;
    end else
    if PtInRect(Rect(CornerOfs, 0, Width - CornerOfs, 3), HitPt) then
    begin
      Msg.Result := htTop;
    end else
    if PtInRect(Rect(Width - CornerOfs, 0, Width, 3), HitPt) or
       PtInRect(Rect(Width - 3, 0, Width, CornerOfs), HitPt) then
    begin
      Msg.Result := htTopRight;
    end else
    if PtInRect(Rect(Width - 3, CornerOfs, Width, Height - CornerOfs), HitPt) then
    begin
      Msg.Result := htRight;
    end else
    if PtInRect(Rect(Width - 3, Height - CornerOfs, Width, Height), HitPt) or
       PtInRect(Rect(Width - CornerOfs, Height - 3, Width, Height), HitPt) then
    begin
      Msg.Result := htBottomRight;
    end else
    if PtInRect(Rect(CornerOfs, Height - 3, Width - CornerOfs, Height), HitPt) then
    begin
      Msg.Result := htBottom;
    end else
    if PtInRect(Rect(0, Height - CornerOfs, 3, Height), HitPt) or
       PtInRect(Rect(0, Height - 3, CornerOfs, Height), HitPt) then
    begin
      Msg.Result := htBottomLeft;
    end else
    if PtInRect(Rect(0, CornerOfs, 3, Height - CornerOfs), HitPt) then
    begin
      Msg.Result := htLeft;
    end;
  end;

  if Msg.Result = 0 then
    inherited;
end;

{---------------------------------------------------------}
{ WM_SIZE - Called whenever the component is resized.     }
{---------------------------------------------------------}
procedure TMILFieldList.WMSize(var Msg: TWMSize);
begin
  if Assigned(OnResize) then
    OnResize(Self);
end;

{---------------------------------------------------------}
{ Register - Component and Property editor registration   }
{---------------------------------------------------------}
procedure Register;
begin
  // Property Editor Registration
  RegisterPropertyEditor(TypeInfo(TAboutProperty), TMILFieldList,
    'About', TAboutProperty);

  // Component Registration
  RegisterComponents('Millennial', [TMILFieldList]);
end;

end.
